@@ -1,7 +1,7 @@
 # -*- coding: iso-8859-1 -*-
 #
 # quakepy/QPCore.py
-# $Id: QPCore.py 159 2009-02-17 13:34:05Z fab $
+# $Id: QPCore.py 253 2009-11-23 15:35:10Z fab $
 #
 # The QuakePy package
 # http://www.quakepy.org
@@ -32,8 +32,8 @@ The QuakePy package
 http://www.quakepy.org
 """
 
-__version__  = '$Id: QPCore.py 159 2009-02-17 13:34:05Z fab $'
-__revision__ = '$Revision: 159 $'
+__version__  = '$Id: QPCore.py 253 2009-11-23 15:35:10Z fab $'
+__revision__ = '$Revision: 253 $'
 __author__   = "Fabian Euchner <fabian@fabian-euchner.de>, Danijel Schorlemmer <ds@usc.edu>"
 __license__  = "GPL"
 
@@ -55,11 +55,14 @@ from QPElement import *
 from QPUtils import *
 from QPDateTime import *
 
+
 POS_TAGNAME, POS_ATTRS, POS_CHILDREN = range(3)
 
+
 class QPObjectList( list ):
     pass
 
+
 class QPObject( object ):
     """
     QuakePy: QPObject
@@ -180,14 +183,16 @@ class QPObject( object ):
         # if no unequal comparison so far, return True
         return True
 
+
     def __ne__( self, T ):
       
         if self.__eq__( T ) is True:
             return False
         else:
             return True
-        
-    def compareEqualBasicType( self, T, varname, pytype, verbose = False ):
+
+
+    def compareEqualBasicType( self, T, varname, pytype, verbose=False ):
         """
         compare basic attributes 'varname' of self and another instance
         """
@@ -259,6 +264,7 @@ class QPObject( object ):
                 attr.append( [ element.xmlname, element.varname, element.pytype ] )
         return attr
 
+
     def _getXMLElementNames( self, vartype ):
         """
         get object attributes with 'element' xmltype from elements list
@@ -272,6 +278,7 @@ class QPObject( object ):
                 elem.append( [ element.xmlname, element.varname, element.pytype ] )
         return elem
 
+
     def _getXMLCDATAName( self ):
         """
         get object attribute with 'cdata' xmltype from elements list
@@ -284,6 +291,7 @@ class QPObject( object ):
                 return [ element.varname, element.pytype ]
         return [ None, None ]
 
+
     def _getXMLExtensionElements( self, elementList ):
         """
         get extension elements from a given element list
@@ -294,11 +302,13 @@ class QPObject( object ):
             if ( ( element.parentaxis is not None ) and ( element.parenttype is not None ) ):
                 elem.append( element )
         return elem
-    
+
+
     def _initMultipleElements( self ):
         for xmlname, varname, pytype in self._getXMLElementNames( 'multiple' ):
             self.__dict__[varname] = QPObjectList()
-            
+
+
     def _getXMLSerializationString( self, attribute ):
         """
         get string for XML serialization of an attribute
@@ -318,8 +328,8 @@ class QPObject( object ):
             return unicode( attribute )
 
     # ------------------------------------------------------------------------
-    
-    def fromXML( self, tree, additionalElements = None ):
+
+    def fromXML( self, tree, additionalElements=None ):
         """
         populate class attributes as defined in 'elements' list of the object
         from pyRXP XML tuple tree
@@ -426,6 +436,7 @@ class QPObject( object ):
 
         return True
 
+
     def toXML( self, tagname, stream ):
         """
         create XML representation for elements given in 'element' tuple
@@ -466,8 +477,9 @@ class QPObject( object ):
                 for tmp in self.__dict__[varname]:
                     tmp.toXML( xmlname, stream )
 
-        # add 'unknown' elements from self.childXMLTree
-        pyrxpTupleTree2XML( self.childXMLTree, stream )
+        # add non-standard elements from self.childXMLTree
+        for curr_extension_node in self.childXMLTree:
+            pyrxpTupleTree2XML( curr_extension_node, stream )
 
         ## add CDATA
         ## note: there should by only one CDATA element in self.elements
@@ -511,7 +523,8 @@ class QPObject( object ):
             error_str = "QPObject::add - parent class %s has no attribute %s" % \
                         ( parentObject.__class__.__name__, parentVariableName )
             raise TypeError, error_str
-                
+
+
     def addObject( self, obj, obj_elements ):
         """
         obj has to have obj.elements for own contents
@@ -522,7 +535,8 @@ class QPObject( object ):
         # add object to instance
         self.__dict__[obj_elements.varname] = obj
 
-    def setElementAxis( self, parentAxis, elementName = None ):
+
+    def setElementAxis( self, parentAxis, elementName=None ):
 
         if parentAxis is not None:
 
@@ -531,8 +545,6 @@ class QPObject( object ):
             else:
                 self.elementAxis = parentAxis
 
-    # ------------------------------------------------------------------------
-
 # ----------------------------------------------------------------------------
 
 class QPPublicObject( QPObject ):
@@ -548,14 +560,16 @@ class QPPublicObject( QPObject ):
     #  'short':   'myname' or '2008-07-26T15:00:00'
     #  'numeric': '152763'
     publicIDStyle = 'full'
-    
+
+
     def __init__( self, publicID = None, **kwargs ):
         super( QPPublicObject, self ).__init__( **kwargs )
         QPPublicObject.publicObjectCtr += 1
         
         self.publicID = publicID
 
-    def createPublicID( self, name = None, **kwargs ):
+
+    def createPublicID( self, name=None, **kwargs ):
         """
         create publicID for a catalog object
 
@@ -587,7 +601,7 @@ class QPPublicObject( QPObject ):
             if name is not None:
                 id = str( name )
             else:
-                id = mxDateTime2ISO( curr_time, secondsdigits = self.secondsDigits )
+                id = mxDateTime2ISO( curr_time, secondsdigits=self.secondsDigits )
 
         else:
             if not name:
@@ -596,19 +610,19 @@ class QPPublicObject( QPObject ):
             id = ''.join( ( 'smi:local/',
                              name,
                              '/',
-                             mxDateTime2ISO( curr_time, secondsdigits = self.secondsDigits ) ) )
+                             mxDateTime2ISO( curr_time, secondsdigits=self.secondsDigits ) ) )
             
         return id
 
+
+    @classmethod
     def getPublicObjectCtr( cls ):
         return cls.publicObjectCtr
 
-    getPublicObjectCtr = classmethod( getPublicObjectCtr )
-    
+
+    @classmethod
     def setPublicIDStyle( cls, style ):
 
         # set publicIDStyle only if it is a valid entry
         if style in ( 'full', 'short', 'numeric' ):
-            cls.publicIDStyle = style
-
-    setPublicIDStyle = classmethod( setPublicIDStyle )
\ No newline at end of file
+            cls.publicIDStyle = style
\ No newline at end of file
