@@ -1,7 +1,7 @@
 # -*- coding: iso-8859-1 -*-
 #
 # quakepy/QPUtils.py
-# $Id: QPUtils.py 198 2009-04-28 16:52:11Z fab $
+# $Id: QPUtils.py 253 2009-11-23 15:35:10Z fab $
 #
 # The QuakePy package
 # http://www.quakepy.org
@@ -32,8 +32,8 @@ The QuakePy package
 http://www.quakepy.org
 """
 
-__version__  = '$Id: QPUtils.py 198 2009-04-28 16:52:11Z fab $'
-__revision__ = '$Revision: 198 $'
+__version__  = '$Id: QPUtils.py 253 2009-11-23 15:35:10Z fab $'
+__revision__ = '$Revision: 253 $'
 __author__   = "Fabian Euchner <fabian@fabian-euchner.de>, Danijel Schorlemmer <ds@usc.edu>"
 __license__  = "GPL"
 
@@ -42,7 +42,7 @@ import datetime
 import re
 import math
 import numpy
-import urllib
+import urllib2
 import gzip, bz2
 import cPickle
 import types
@@ -52,40 +52,72 @@ import colorsys
 
 from lxml import etree
 
-from mx.DateTime     import Date, DateTimeType, DateTimeDeltaFromSeconds, DateTimeDelta, DateTimeDeltaFrom, TimeDelta
+from mx.DateTime import Date, DateTime, DateTimeType
+from mx.DateTime import DateTimeDeltaFromSeconds, DateTimeFromAbsDays, DateTimeDelta, DateTimeDeltaFrom, TimeDelta
 from mx.DateTime.ISO import ParseDateTimeUTC
 
+POS_TAGNAME, POS_ATTRS, POS_CHILDREN = range(3)
+
+# -------------------------------------------------------------------------------------
+
 def getQPDataSource( filename, **kwargs ):
 
     if filename.startswith('http://') or filename.startswith('https://') or filename.startswith('ftp://'):
+
+        request = urllib2.Request( filename )
+        opener  = urllib2.build_opener()
+        
         try:
-            file_object = urllib.urlopen( filename )
+            file_object = opener.open( request )
         except:
-            raise IOError, "getQPDataSource: cannot open data source"
+            raise IOError, "getQPDataSource: cannot open data source from URL %s" % filename
+
+        if 'compression' in kwargs.keys():
+
+            file_contents = file_object.read()
+            
+            if kwargs['compression'] == 'gz':
+
+                try:
+                    file_object = gzip.GzipFile( fileobj=cStringIO.StringIO( file_contents ) )
+                except:
+                    raise IOError, "getQPDataSource: cannot open data source from gzipped stream, URL %s" % filename
+
+            elif kwargs['compression'] == 'bz2':
+
+                try:
+                    file_object = cStringIO.StringIO( bz2.decompress( file_contents ) )
+                except:
+                    raise IOError, "getQPDataSource: cannot open data source from b2zipped stream, URL %s" % filename
+
     else:
-        if ( 'compression' in kwargs.keys() ):
-            if ( kwargs['compression'] == 'gz' ):
+        if 'compression' in kwargs.keys():
+
+            if kwargs['compression'] == 'gz':
                 try:
                     file_object = gzip.GzipFile( filename )
                 except:
-                    raise IOError, "getQPDataSource: cannot open data source"
-            elif ( kwargs['compression'] == 'bz2' ):
+                    raise IOError, "getQPDataSource: cannot open gzipped data source file %s" % filename
+
+            elif kwargs['compression'] == 'bz2':
                 try:
                     file_object = bz2.BZ2File( filename )
                 except:
-                    raise IOError, "getQPDataSource: cannot open data source"
+                    raise IOError, "getQPDataSource: cannot open b2zipped data source file %s" % filename
         else:
-            if ( 'binary' in kwargs.keys() ) and ( kwargs['binary'] == True ):
+            if ( 'binary' in kwargs.keys() ) and ( kwargs['binary'] is True ):
                 open_flag = 'rb'
             else:
                 open_flag = 'r'
+
             try:
                 file_object = open( filename, open_flag )
             except:
-                raise IOError, "getQPDataSource: cannot open data source"
+                raise IOError, "getQPDataSource: cannot open data source file %s" % filename
                 
     return file_object
 
+
 def writeQPData( filename, **kwargs ):
 
     if ( 'compression' in kwargs.keys() ):
@@ -103,7 +135,7 @@ def writeQPData( filename, **kwargs ):
         else:
             raise IOError, "writeQPData: no valid compression format given"
     else:
-        if ( 'binary' in kwargs.keys() ) and ( kwargs['binary'] == True ):
+        if ( 'binary' in kwargs.keys() ) and ( kwargs['binary'] is True ):
             open_flag = 'wb'
         else:
             open_flag = 'w'
@@ -114,6 +146,7 @@ def writeQPData( filename, **kwargs ):
 
     return file_object
 
+# -------------------------------------------------------------------------------------
 
 def xmlPrettyPrint( input_data, output, **kwargs ):
     """
@@ -143,7 +176,66 @@ def xmlPrettyPrint( input_data, output, **kwargs ):
         ostream.write( txt_pretty )
     except:
         raise IOError, "QPUtils:xmlPrettyPrint() - cannot write to file"
-    
+
+
+def pyrxpTupleTree2XML( node, stream ):
+
+    if node is None:
+        return
+
+    # is current node a simple string?
+    if isinstance( node, basestring ):
+        stream.write( node )
+        return
+
+    # check if current node has element name
+    try:
+        elementname = node[POS_TAGNAME]
+    except:
+        elementname = ""
+
+    # if node has no element name, output the children
+    if len( elementname.strip() ) == 0:
+
+        for child in node[POS_CHILDREN]:
+            pyrxpTupleTree2XML( child, stream )
+        return
+
+    # regular element with element name
+    stream.write( ''.join( ( "<", elementname ) ) )
+
+    # set attributes
+    try:
+
+        # loop over attribute/value pairs
+        for attributeName in node[POS_ATTRS].keys():
+            stream.write( " %s=\"%s\"" % ( attributeName, node[POS_ATTRS][attributeName] ) )
+
+    except AttributeError, IndexError:
+
+        # if there are no attributes, do nothing
+        pass
+
+    if (    ( node[POS_CHILDREN] is None )
+         or ( node[POS_CHILDREN] is not None and len( node[POS_CHILDREN] ) == 0 ) ):
+
+        # node has no further children, close tag
+        stream.write( "/>" )
+    else:
+
+        # node has children, close starting tag and insert child nodes
+        stream.write( ">" )
+
+        for child in node[POS_CHILDREN]:
+            pyrxpTupleTree2XML( child, stream )
+
+        # write closing tag
+        stream.write( "</%s>" % elementname )
+
+    return
+
+# -------------------------------------------------------------------------------------
+
 def pickleObj( obj, file, **kwargs ):
     fh = writeQPData( file, binary = True, **kwargs )
     try:
@@ -154,6 +246,7 @@ def pickleObj( obj, file, **kwargs ):
     fh.close()
     return True
 
+
 def unpickleObj( file, **kwargs ):
     fh = getQPDataSource( file, binary = True, **kwargs )
     try:
@@ -164,8 +257,6 @@ def unpickleObj( file, **kwargs ):
     fh.close()
     return obj
 
-def pyrxpTupleTree2XML( child, stream ):
-    pass
 
 class picklable_boundmethod( object ):
     """
@@ -183,6 +274,7 @@ class picklable_boundmethod( object ):
     def __call__( self, *a, **kw ):
         return self.mt( *a, **kw )
 
+
 class Callable( object ):
     """
     from ASPN Python Cookbook
@@ -192,6 +284,7 @@ class Callable( object ):
     def __init__( self, anycallable ):
         self.__call__ = anycallable
 
+
 def addUnique( baseList, otherList ):
     """
     Python Cookbook, O'Reilly, Recipe 5.12
@@ -206,6 +299,8 @@ def addUnique( baseList, otherList ):
             baseList.append( item )
             auxSet.add( item )
 
+# -------------------------------------------------------------------------------------
+
 class CombinationsNumPy( object ):
     """
     holds lists for m-element combination of index vector of length N
@@ -244,7 +339,8 @@ class CombinationsNumPy( object ):
             
         if not ( 'nocompute' in kwargs.keys() and kwargs['nocompute'] is True ):
             self.compute()
-    
+
+
     def getCombinations( self, m, n ):
         
         if n > self.N or n < 2:
@@ -271,7 +367,8 @@ class CombinationsNumPy( object ):
                   
         else:
             raise ValueError, "CombinationsNumPy::getCombinations - illegal m"
-        
+
+
     def compute( self ):
 
         for m in self.mRange:
@@ -313,6 +410,7 @@ class CombinationsNumPy( object ):
                         del temp_list[tmp_entry_idx]
                         added = added + 1
 
+
 class Combinations( object ):
     """
     holds lists for m-element combination of index vector of length N
@@ -351,7 +449,8 @@ class Combinations( object ):
             
         if not ( 'nocompute' in kwargs.keys() and kwargs['nocompute'] is True ):
             self.compute()
-             
+
+
     def getCombinations( self, m, n ):
         
         if n > self.N or n < 2:
@@ -378,7 +477,8 @@ class Combinations( object ):
                   
         else:
             raise ValueError, "Combinations::getCombinations - illegal m"
-        
+
+
     def compute( self ):
 
         for m in self.mRange:
@@ -416,6 +516,7 @@ class Combinations( object ):
                         # delete matching entry from un-ordered list
                         del temp_list[tmp_entry_idx]
 
+
 def xuniqueCombinations(items, n):
     """
     from ASPN Python Cookbook
@@ -426,12 +527,14 @@ def xuniqueCombinations(items, n):
     See also: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66463
     See also: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66465
     """
-    if n==0: yield []
+    if n == 0:
+        yield []
     else:
         for i in xrange(len(items)):
             for cc in xuniqueCombinations(items[i+1:],n-1):
                 yield [items[i]]+cc
 
+
 def xuniqueCombinations_2(items, n):
     """
     from ASPN Python Cookbook
@@ -442,12 +545,15 @@ def xuniqueCombinations_2(items, n):
     See also: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66463
     See also: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66465
     """
-    if n==0: yield []
+    if n == 0:
+        yield []
     else:
         for i in xrange(len(items)-n+1):
             for cc in xuniqueCombinations_2(items[i+1:],n-1):
                 yield [items[i]]+cc
 
+# -------------------------------------------------------------------------------------
+
 def distanceBetweenPoints( p1, p2, lat_for_londist = None ):
     """
     computes distance between two geographical points
@@ -481,6 +587,7 @@ def distanceBetweenPoints( p1, p2, lat_for_londist = None ):
     distanceXYZ = math.sqrt( math.pow( distanceXY, 2 ) + math.pow( p1_dep - p2_dep, 2) )
     return ( distanceXYZ, distanceXY )
 
+
 def horizontalErrorFromLatLon( lat_err, lon_err, latitude ):
     """
     compute horizontal error in km from errors on lat/lon given in degrees
@@ -491,84 +598,7 @@ def horizontalErrorFromLatLon( lat_err, lon_err, latitude ):
     horizontal_error = math.sqrt( math.pow( lat_err_km, 2 ) + math.pow( lon_err_km, 2) )
     return horizontal_error
 
-#def gmtColormap( fileName, GMTPath = None ):
-    #"""
-    #from Matplotlib Cookbook on scipy.org
-    #http://www.scipy.org/Cookbook/Matplotlib/Loading_a_colormap_dynamically
-    #"""
-    
-    #N = Numeric
-    #if type(GMTPath) == type(None):
-        #filePath = "/usr/local/cmaps/"+ fileName+".cpt"
-    #else:
-        #filePath = GMTPath+"/"+ fileName +".cpt"
-    #try:
-        #f = open(filePath)
-    #except:
-        #print "file ",filePath, "not found"
-        #return None
-
-    #lines = f.readlines()
-    #f.close()
-
-    #x = []
-    #r = []
-    #g = []
-    #b = []
-    #colorModel = "RGB"
-    #for l in lines:
-        #ls = l.split()
-        #if l[0] == "#":
-            #if ls[-1] == "HSV":
-                #colorModel = "HSV"
-                #continue
-            #else:
-                #continue
-        #if ls[0] == "B" or ls[0] == "F" or ls[0] == "N":
-            #pass
-        #else:
-            #x.append(float(ls[0]))
-            #r.append(float(ls[1]))
-            #g.append(float(ls[2]))
-            #b.append(float(ls[3]))
-            #xtemp = float(ls[4])
-            #rtemp = float(ls[5])
-            #gtemp = float(ls[6])
-            #btemp = float(ls[7])
-
-    #x.append(xtemp)
-    #r.append(rtemp)
-    #g.append(gtemp)
-    #b.append(btemp)
-
-    #nTable = len(r)
-    #x = N.array( x , N.Float)
-    #r = N.array( r , N.Float)
-    #g = N.array( g , N.Float)
-    #b = N.array( b , N.Float)
-    #if colorModel == "HSV":
-        #for i in range(r.shape[0]):
-            #rr,gg,bb = colorsys.hsv_to_rgb(r[i]/360.,g[i],b[i])
-            #r[i] = rr ; g[i] = gg ; b[i] = bb
-    #if colorModel == "HSV":
-        #for i in range(r.shape[0]):
-            #rr,gg,bb = colorsys.hsv_to_rgb(r[i]/360.,g[i],b[i])
-            #r[i] = rr ; g[i] = gg ; b[i] = bb
-    #if colorModel == "RGB":
-        #r = r/255.
-        #g = g/255.
-        #b = b/255.
-    #xNorm = (x - x[0])/(x[-1] - x[0])
-
-    #red = []
-    #blue = []
-    #green = []
-    #for i in range(len(x)):
-        #red.append([xNorm[i],r[i],r[i]])
-        #green.append([xNorm[i],g[i],g[i]])
-        #blue.append([xNorm[i],b[i],b[i]])
-    #colorDict = {"red":red, "green":green, "blue":blue}
-    #return (colorDict)
+# -------------------------------------------------------------------------------------
 
 def mxDateTime2ISO( date_arr, **kwargs ):
     """
@@ -691,6 +721,7 @@ def mxDateTime2ISO( date_arr, **kwargs ):
     else:
         return new_arr[0]
 
+
 def decimalYear( datetime ):
     """
     return (floating point) decimal year representation of a mx.DateTime input value
@@ -721,7 +752,32 @@ def decimalYear( datetime ):
     # return decimal year as sum of integer year (AD) and fraction of year
     # computed above
     return datetime.year + year_fraction
-    
+
+
+def fromDecimalYear( decimalyear ):
+    """
+    return mx.DateTime object corresponding to (floating point) decimal year
+
+    mx.DateTime provides:
+     .absdate - integer days since epoch
+     .abstime - (fractional) seconds from beginning (time 00:00:00) of day (no leap seconds)
+     .absdays - (fractional) days since epoch
+    """
+
+    # get year = integer fraction of decimalyear
+    # NOTE: if decimal year is very close to the next higher integer value, rounding takes place
+    year_fraction, year = math.modf( decimalyear )
+    startyear_dt = DateTime( int( year ), 1, 1 )
+
+    # get seconds that have passed in fraction of the current year
+    if startyear_dt.is_leapyear:
+        year_seconds = year_fraction * 86400.0 * 366
+    else:
+        year_seconds = year_fraction * 86400.0 * 365
+
+    return startyear_dt + DateTimeDeltaFromSeconds( year_seconds )
+
+
 def fixTimeComponents( hour, minute, second ):
     """
     in time strings with the format HH:MM:SS[.ss...], values are sometimes HH=24, MM=60, and SS=60
@@ -751,6 +807,7 @@ def fixTimeComponents( hour, minute, second ):
     return { 'component'    : ( checked_hour, checked_min, checked_sec ),
              'increaseFlag' : ( increaseDay, increaseHour, increaseMin ) }
 
+
 def adjustDateTime( adjustFlags, dateTime ):
     """
     sometimes a DateTime object has to be adjusted, when illegal components were specified
@@ -762,19 +819,21 @@ def adjustDateTime( adjustFlags, dateTime ):
     """
 
     # add one day
-    if adjustFlags[0] == True:
+    if adjustFlags[0] is True:
         dateTime += DateTimeDelta( 1.0 )
 
     # add one hour
-    if adjustFlags[1] == True:
+    if adjustFlags[1] is True:
         dateTime += TimeDelta( 1.0 )
 
     # add one minute
-    if adjustFlags[2] == True:
+    if adjustFlags[2] is True:
         dateTime += DateTimeDeltaFromSeconds( 60.0 )
         
     return dateTime
 
+# -------------------------------------------------------------------------------------
+
 def normalizeFloat( value ):
     """
     compute components of scientific notation in normalized format
@@ -785,6 +844,7 @@ def normalizeFloat( value ):
     mantissa = float( value / (10**exponent) )
     return ( mantissa, exponent )
 
+
 def exponentialFloatFromString( mantissa, exponent ):
     """
     create floating point number from mantissa and exponent without computation
@@ -806,6 +866,8 @@ def exponentialFloatFromString( mantissa, exponent ):
 
     return float( 'e'.join( ( str(mantissa), str(exponent) ) ) )
 
+# -------------------------------------------------------------------------------------
+
 def frange( start, end = None, inc = 1.0 ):
     """
     borrowed from Python Cookbook (O'Reilly), Recipe 19.1
@@ -815,7 +877,7 @@ def frange( start, end = None, inc = 1.0 ):
            frange(5.0, 10.0)      - [ 5.0, 6.0, 7.0, 8.0, 9.0, 10.0 ]
            frange(5.0, 15.0, 5.0) - [ 5.0, 10.0, 15.0 ]
     """
-    if end == None:
+    if end is None:
         # ensure that end is a floating point number
         end = start + 0.0
         start = 0.0
@@ -828,6 +890,108 @@ def frange( start, end = None, inc = 1.0 ):
     nitems = int( round( (end - start) / inc ) ) + 1
     return numpy.arange( nitems ) * inc + start
 
+
+def remove_ij( x, i=None, j=None ):
+    """
+    from http://www.scipy.org/PerformanceTips
+    Removing the i-th row and j-th column of a 2d array or matrix (fast way, without copies)
+    Author: Keith Goodman (?)
+
+    operates on numpy array x
+    """
+
+    ## Row i and column j divide the array into 4 quadrants
+
+    # full array, including upper left quadrant
+    y = x[:-1,:-1]
+
+    if j is None:
+        
+        # lower part
+        y[i:,:] = x[i+1:,:]
+
+    elif i is None:
+
+        # right part
+        y[:,j:] = x[:,j+1:]
+        
+    else:
+
+        # upper right quadrant
+        y[:i,j:] = x[:i,j+1:]
+
+        # lower left quadrant
+        y[i:,:j] = x[i+1:,:j]
+
+        # lower right quadrant
+        y[i:,j:] = x[i+1:,j+1:]
+    
+    return y
+
+
+def locateInArray( array, value ):
+    """
+    locate floating-point value 'value' in list or tuple of floats 'array'
+    return zero-offset array index 'idx'
+
+    returns idx so that array[idx] <= value <  array[idx+1] for idx in 0...len(array)-3
+                        (value is in bin of idx, upper bound of bin excluded)
+
+                        array[idx] <= value <= array[idx+1] for idx=len(array)-2
+                        (value is in bin of idx, upper bound included, if value is in last array bin)
+
+                        idx=-1:             value is smaller than lower array bound
+                        idx=len(array)-1:   value is larger than upper array bound
+
+    built after Numerical Recipes 'locate' routine
+    """
+
+    n          = len( array )
+    checkLower = 0
+    checkUpper = n+1
+
+    if array[n-1] >= array[0]:
+        ascending = True
+    else:
+        ascending = False
+
+    while ( ( checkUpper - checkLower ) > 1 ):
+
+        checkMean = ( checkUpper + checkLower ) // 2
+
+        if ( ( value >= array[checkMean-1] ) is ascending ):
+            checkLower = checkMean
+        else:
+            checkUpper = checkMean
+
+    if checkLower == 0:
+
+        # out of range at bottom of array
+        idx = -1
+
+    else:
+
+        if checkLower == n:
+
+            # we are either on last array value or out of upper bound
+            if floatEqual( value, array[n-1] ):
+
+                # exception: exactly last array value
+                idx = n-2
+
+            else:
+
+                # out of range at top of array
+                idx = n-1
+
+        else:
+            # standard case: return zero-offset bin idx
+            idx = checkLower - 1
+
+    return idx
+
+# -------------------------------------------------------------------------------------
+
 def floatEqual( f1, f2, epsilon = 1e-12 ):
     """
     checks if two floating point values can be considered equal
@@ -838,6 +1002,7 @@ def floatEqual( f1, f2, epsilon = 1e-12 ):
     else:
         return True
 
+
 def floatEqualPairs( pairs, epsilon = 1e-12 ):
     """
     checks for a sequence of floating point value pairs if they can be considered equal
@@ -850,7 +1015,8 @@ def floatEqualPairs( pairs, epsilon = 1e-12 ):
             raise ValueError, "QPUtils:floatEqualPairs - pair %s and %s are not equal" % ( pair[0], pair[1] )
 
     return True
-    
+
+
 def equalPairs( pairs ):
     """
     checks for a sequence of integer/string value pairs if they are equal
@@ -864,63 +1030,9 @@ def equalPairs( pairs ):
 
     return True
 
-def flocate( grid, test, idx ):
-    """
-    TODO:
-    according to Numerical Recipes 'locate' code
-    """
-    
-    #void ffnr::fflocate(double* xx, int n, double x, int  *j)
-#{
-  #//
-  #// xx[0..n-1] normales C-Array (zero offset)
-  #// x zu locatender Wert
-  #// zurueckgelieferte Indizes (ohne Ueberschr.): 0 - n-2
-  #// genau obere Grenze (xx[n-1])               : n-2 (statt n-1)
-  #//
-  #// bei Ueberschreitung unten: j=n-1
-  #// bei Ueberschreitung oben:  j=n
-  #//
-  #// liefert j, so dass   xx[j] <= x <  x[j+1] fuer j=0,...,n-3
-  #// j = 0 - n-3   :      xx[j] <= x <  x[j+1]
-  #// Ausnahme j=n-2:      xx[j] <= x <= x[j+1]
-  #// 
-  #//
-  
-  #int ju, jm, jl;
-  #int ascnd;
-  #jl = 0;
-  #ju = n+1;
-  #ascnd = (xx[n-1] >= xx[0]);             /* auf- oder absteigend? */
-  
-  #while (ju-jl > 1) {                     /* Indizes nicht direkt benachbart */
-    #jm=(ju+jl) >> 1;                      /* Teilt durch zwei mit Rest */
-                                          #/* mittlerer Index */
-
-    #//
-    #// Achtung: hier Zugriff auf Index jm-1 wg. zero offset
-    #//
-    #if (x >= xx[jm-1] == ascnd)
-      #jl = jm;                           /* groesser als Mittel */
-    #else
-      #ju = jm;                           /* kleiner als Mittel */
-  #}
-  
-  #if (jl == 0) 
-    #*j = n-1;                   /* out of range unten liefert n-1 */
-  #else 
-    #if (jl == n) {              /* letzter oder out of range oben */
-      #if (x == xx[n-1])
-    #*j = n-2;               /* Besonderheit! letzter Wert bekommt n-2 */
-      #else 
-    #*j = n;                 /* out of range oben liefert n */
-    #}
-    #else
-      #*j = jl-1;                /* Normalfall: gib ZERO-offset-Index zurueck */
-#}
-    pass
-
+# -------------------------------------------------------------------------------------
 
+## functions to determine memory usage
 
 _proc_status = '/proc/%d/status' % os.getpid( )
 _scale = {'kB': 1024.0, 'mB': 1024.0*1024.0,
