@@ -1,7 +1,7 @@
 # -*- coding: iso-8859-1 -*-
 #
 # quakepy/pmc/PMCInventory.py
-# $Id: PMCInventory.py 186 2009-03-31 22:23:33Z fab $
+# $Id: PMCInventory.py 248 2009-11-04 12:28:15Z fab $
 #
 # The QuakePy package
 # http://www.quakepy.org
@@ -32,8 +32,8 @@ The QuakePy package
 http://www.quakepy.org
 """
 
-__version__  = '$Id: PMCInventory.py 186 2009-03-31 22:23:33Z fab $'
-__revision__ = '$Revision: 186 $'
+__version__  = '$Id: PMCInventory.py 248 2009-11-04 12:28:15Z fab $'
+__revision__ = '$Revision: 248 $'
 __author__   = "Danijel Schorlemmer <ds@usc.edu>, Fabian Euchner <fabian@fabian-euchner.de>"
 __license__  = "GPL"
 
@@ -55,21 +55,43 @@ from   xml.sax   import saxutils
 
 sys.path.append('..')
 
-from QPCore import *
+from QPCore        import *
+from PMCFunctions  import *
+
 from WaveformStreamID import WaveformStreamID
 
+
 POS_TAGNAME, POS_ATTRS, POS_CHILDREN = range(3)
 
+
 class PMCStation( QPObject ):
     
     def __init__( self, dist_style = None, **kwargs ):
 
         # holds objects of type 'PMCChannel'
-        self.channels     = []
+        self.channels = []
+
+        # onTime per station = time intervals in which at least one channel was operating
+        self.onTime = []
 
         # PMCProbabilityDistribution object, holds pickInfo and distro
         self.distribution = PMCProbabilityDistribution( dist_style, **kwargs )
 
+        # start and end date of catalog used for pickInfo
+        self.catalogStartDate      = None   # type QPDateTime
+        self.catalogEndDate        = None   # type QPDateTime
+        
+        # inforamtion for refined picks
+        self.onTimesAreRefinedFlag = False
+        
+        self.onTimeRefined         = None
+        self.interPickTime         = None   # list, times between seen arrivals at station, in days 
+        
+        self.meanInterPickTime     = None   # mean time between seen arrivals at station, in days
+        self.timeIntervalScale     = None
+
+        self.ontimeMinPickCtr      = None
+        
     ## -----------------------------------------------------------------------
     
     def fromXML( self, tree, additionalElements = None ):
@@ -94,6 +116,30 @@ class PMCStation( QPObject ):
                 cha.fromXML( child, additionalElements )
                 self.channels.append( cha )
 
+            elif child[POS_TAGNAME] == 'onTime':
+
+                # onTime element has 2 required attributes, 'start' and 'end'
+                # convert to QPDateTime objects
+                attr_dict = child[POS_ATTRS]
+                self.onTime.append( [ QPDateTime( attr_dict['start'] ),
+                                      QPDateTime( attr_dict['end'] ) ] )
+                                             
+            elif child[POS_TAGNAME] == 'onTimeRefined':
+
+                if self.onTimeRefined is None:
+                    self.onTimeRefined = []
+                    self.onTimesAreRefinedFlag = True
+
+                attr_dict = child[POS_ATTRS]
+                self.onTimeRefined.append( [ QPDateTime( attr_dict['start'] ),
+                                             QPDateTime( attr_dict['end'] ) ] )
+
+            elif child[POS_TAGNAME] == 'interPickTime':
+
+                inter_pick_times_list = map( float, child[POS_CHILDREN].pop().split() )
+                if len( inter_pick_times_list ) > 0:
+                    self.interPickTime = inter_pick_times_list
+
             # children of complex type
             elif child[POS_TAGNAME] == 'PMCProbabilityDistribution':
                 self.distribution = PMCProbabilityDistribution()
@@ -111,52 +157,114 @@ class PMCStation( QPObject ):
                 #description_sanitized = saxutils.unescape( child[POS_CHILDREN].pop() )
 
                 self.description = description_sanitized
-                
-    def toXML( self, tagname, stream ):
 
-        stream.writelines( [ '<', tagname ] )
+            elif child[POS_TAGNAME] == 'catalogStartDate':
+
+                self.catalogStartDate = QPDateTime( child[POS_CHILDREN].pop() )
+
+            elif child[POS_TAGNAME] == 'catalogEndDate':
+
+                self.catalogEndDate = QPDateTime( child[POS_CHILDREN].pop() )
+                    
+            elif child[POS_TAGNAME] == 'meanInterPickTime':
+
+                self.meanInterPickTime = float( child[POS_CHILDREN].pop() )
+
+            elif child[POS_TAGNAME] == 'timeIntervalScale':
+
+                self.timeIntervalScale = float( child[POS_CHILDREN].pop() )
+
+            elif child[POS_TAGNAME] == 'ontimeMinPickCtr':
+
+                self.ontimeMinPickCtr = int( child[POS_CHILDREN].pop() )
+                
+
+    def toXML( self, tagname, stream, **kwargs ):
+        """
+        kwargs: noStationChildElements=True: write only <PMCStation> tag with attributes (default: False) 
+        """
         
+        stream.writelines( [ '<', tagname ] )
+
         # XML attributes: networkCode, stationCode, latitude, longitude, elevation (mandatory)
         stream.write( ''.join( (' networkCode="', self.networkCode, '"') ) )
         stream.write( ''.join( (' stationCode="', self.stationCode, '"') ) )
 
         if hasattr( self, 'locationCode' ) and self.locationCode is not None:
             stream.write( ''.join( (' locationCode="', self.locationCode, '"') ) )
-            
+
         stream.write( ''.join( (' latitude="', str(self.latitude), '"') ) )
         stream.write( ''.join( (' longitude="', str(self.longitude), '"') ) )
         stream.write( ''.join( (' elevation="', str(self.elevation), '"') ) )
-        
+
         stream.write( '>' )
 
-        # children of basic type
-        if hasattr( self, 'description' ) and self.description:
+        if not ( 'noStationChildElements' in kwargs.keys() and kwargs['noStationChildElements'] is True ):
+            
+            # children of basic type
+            if hasattr( self, 'description' ) and self.description is not None:
 
-            # sanitize description string for XML
-            
-            # self.description is Unicode!
-            description_str_xml = saxutils.escape( self.description ).encode( 'utf-8', 'xmlcharrefreplace' )
+                # sanitize description string for XML
 
-            # ----- this works, for non-unicode self.description
-            #description_str_xml = saxutils.escape( self.description )
-            
-            stream.write( ''.join( ('<description>', description_str_xml, '</description>') ) )
-            
-        # multiple children of same type
-        if hasattr( self, 'channels' ) and self.channels is not None:
+                # self.description is Unicode!
+                description_str_xml = saxutils.escape( self.description ).encode( 'utf-8', 'xmlcharrefreplace' )
+
+                # ----- this works, for non-unicode self.description
+                #description_str_xml = saxutils.escape( self.description )
+
+                stream.write( ''.join( ('<description>', description_str_xml, '</description>') ) )
+
+            if hasattr( self, 'catalogStartDate' ) and self.catalogStartDate is not None:
+
+                stream.write( ''.join( ('<catalogStartDate>', self.catalogStartDate.toISO( showsecfrac=False ), '</catalogStartDate>') ) )
+                
+            if hasattr( self, 'catalogEndDate' ) and self.catalogEndDate is not None:
+
+                stream.write( ''.join( ('<catalogEndDate>', self.catalogEndDate.toISO( showsecfrac=False ), '</catalogEndDate>') ) )
+                
+            if hasattr( self, 'meanInterPickTime' ) and self.meanInterPickTime is not None:
+
+                stream.write( ''.join( ('<meanInterPickTime>', str( self.meanInterPickTime ), '</meanInterPickTime>') ) )
+                
+            if hasattr( self, 'timeIntervalScale' ) and self.timeIntervalScale is not None:
+
+                stream.write( ''.join( ('<timeIntervalScale>', str( self.timeIntervalScale ), '</timeIntervalScale>') ) )
+
+            if hasattr( self, 'ontimeMinPickCtr' ) and self.ontimeMinPickCtr is not None:
+
+                stream.write( ''.join( ('<ontimeMinPickCtr>', str( self.ontimeMinPickCtr ), '</ontimeMinPickCtr>') ) )
+
+            if hasattr( self, 'interPickTime' ) and self.interPickTime is not None and len( self.interPickTime ) > 0:
+
+                stream.write( ''.join( ('<interPickTime>', ' '.join( map( str, self.interPickTime ) ), '</interPickTime>') ) )
+                
+            ## multiple children of same type
+
+            # channels
             for cha in self.channels:
                 cha.toXML( 'PMCChannel', stream )
+                    
+            # onTime
+            for ot in self.onTime:
+                stream.writelines( ''.join( ( '<onTime start="', ot[0].toISO( showsecfrac=False ),
+                                            '" end="', ot[1].toISO( showsecfrac=False ), '"/>' ) ) )
+                                            
+            # onTimeRefined (optional)
+            if hasattr( self, 'onTimeRefined' ) and self.onTimeRefined is not None:
+                for ot in self.onTimeRefined:
+                    stream.writelines( ''.join( ( '<onTimeRefined start="', ot[0].toISO( showsecfrac=False ),
+                                                '" end="', ot[1].toISO( showsecfrac=False ), '"/>' ) ) )
 
-        # children of complex type
-        if hasattr( self, 'distribution' ) and self.distribution is not None:
-            self.distribution.toXML( 'PMCProbabilityDistribution', stream )
+            # children of complex type
+            if hasattr( self, 'distribution' ) and self.distribution is not None:
+                self.distribution.toXML( 'PMCProbabilityDistribution', stream )
 
         stream.writelines( [ '</', tagname, '>' ] )
         return True
 
     ## -----------------------------------------------------------------------
          
-    def fillup( self, calcMagnitudeFromDistance, override = False ):
+    def fillup( self, calcMagnitudeFromDistance, override=False ):
         self.distribution.fillup( calcMagnitudeFromDistance, override )
 
 
@@ -174,10 +282,10 @@ class PMCStationList( QPObject ):
                         
         self._initMultipleElements()
 
-        
 ## ---------------------------------------------------------------------------
     
 class PMCChannel( QPObject ):
+    
     def __init__( self ):
 
         # onTime holds a list of [ startTime, endTime ] pairs
@@ -185,6 +293,7 @@ class PMCChannel( QPObject ):
         self.onTime     = []
         self.waveformID = None
 
+
     def fromXML( self, tree, additionalElements = None ):
 
         for child in tree[POS_CHILDREN]:
@@ -203,6 +312,7 @@ class PMCChannel( QPObject ):
                 self.onTime.append( [ QPDateTime( attr_dict['start'] ),
                                       QPDateTime( attr_dict['end'] ) ] )
 
+
     def toXML( self, tagname, stream  ):
 
         stream.writelines( [ '<', tagname, '>' ] )
@@ -265,7 +375,7 @@ class PMCProbabilityDistribution( QPObject ):
 
     ## -----------------------------------------------------------------------
     
-    def fromXML( self, tree, additionalElements = None ):
+    def fromXML( self, tree, additionalElements=None ):
 
         # XML attributes: distStyle, smooth (mandatory)
         attr_dict      = tree[POS_ATTRS]
@@ -321,7 +431,8 @@ class PMCProbabilityDistribution( QPObject ):
             else:
                 raise ValueError, "PMCInventory::fromXML - no distance and magnitude grids"
 
-    def toXML( self, tagname, stream  ):
+
+    def toXML( self, tagname, stream ):
         stream.writelines( [ '<', tagname ] )
 
         # XML attributes: distStyle, smooth (mandatory)
@@ -363,8 +474,7 @@ class PMCProbabilityDistribution( QPObject ):
 
     ## -----------------------------------------------------------------------
 
-
-    def getProbability( self, magnitude, distance, calcMagnitudeFromDistance, override = False ):
+    def getProbability( self, magnitude, distance, calcMagnitudeFromDistance, override=False ):
         """
         get probability for given magnitude and distance
         if not yet available in grid, compute probability value
@@ -398,23 +508,26 @@ class PMCProbabilityDistribution( QPObject ):
                                                                                dist_idx )
             else:
                 # fill up probability grid with raw probability value (not smoothed)
-                self.distro[dist_idx, mag_idx], dummy = self.calcRawProbability( magnitude,
-                                                                                 distance,
-                                                                                 calcMagnitudeFromDistance )
+                self.distro[dist_idx, mag_idx], dummy = self.calcRawProbability( 
+                                                                            magnitude,
+                                                                            distance,
+                                                                            calcMagnitudeFromDistance )
 
         # return prob. value from distribution Grid
         return self.distro[dist_idx, mag_idx]
+
         
     def _getMagDistIdx( self, magnitude, distance ):
         """
         return tuple of located indices ( mag_idx, dist_idx )
         """
 
-        # locate mag idx in magnitude grid
-        #mag_idx = flocate( magnitude, self.magnitudeGrid ) 
-
-        # locate distance idx in distance grid
-        #dist_idx = flocate( distance, self.distanceGrid )
+        ## locating magnitude and distance in grid: we could
+        ## use QPUtils.locateInArray(), but we compute the index directly
+        ## since it is faster
+        
+        # mag_idx = locateInArray( self.magnitudeGrid, magnitude )
+        # dist_idx = locateInArray( self.distanceGrid, distance )
 
         if self.distStyle == 1:
             mag_idx  = int(round(magnitude * 5))
@@ -436,9 +549,10 @@ class PMCProbabilityDistribution( QPObject ):
             dist_idx = 0
 
         return ( mag_idx, dist_idx )
+
     
     def calcSmoothedProbability( self, calcMagnitudeFromDistance, magnitude, distance,
-                                 mag_idx = None, dist_idx = None ):
+                                 mag_idx=None, dist_idx=None ):
         """
         calc smoothed probability for given magnitude and distance
         """
@@ -454,8 +568,8 @@ class PMCProbabilityDistribution( QPObject ):
             for curr_dist_idx in reversed( xrange( dist_idx, len(self.distanceGrid) ) ):
 
                 if numpy.isnan( self.distro[curr_dist_idx, curr_mag_idx] ):
-                    self.distro[curr_dist_idx, curr_mag_idx], dummy = \
-                             self.calcRawProbability( self.magnitudeGrid[curr_mag_idx],
+                    self.distro[curr_dist_idx, curr_mag_idx], dummy = self.calcRawProbability(
+                                                      self.magnitudeGrid[curr_mag_idx],
                                                       self.distanceGrid[curr_dist_idx],
                                                       calcMagnitudeFromDistance )
 
@@ -480,13 +594,14 @@ class PMCProbabilityDistribution( QPObject ):
                                                                         self.distro[curr_dist_idx, curr_mag_idx-1] )
 
         return self.distro[dist_idx, mag_idx]
+
     
     def calcRawProbability( self, magnitude, distance, calcMagnitudeFromDistance ):
         """
         calc raw probability for given magnitude and distance
         """
 
-        # if pickInfo has no rows, return probabilty 0.0
+        # if pickInfo has no rows, return probability 0.0
         if self.pickInfo.shape[0] == 0:
             return ( 0.0, 0 )
         
@@ -495,8 +610,6 @@ class PMCProbabilityDistribution( QPObject ):
 
         # 1st column: picked or not picked?
         values[:,0] = self.pickInfo[:,2]
-        #values[:,0] = self.pickInfo[:,2].copy()
-        #print values[:,0].sum()
         
         # 2nd column: magnitude differences
         values[:,1] = self.pickInfo[:,1] - magnitude
@@ -508,8 +621,6 @@ class PMCProbabilityDistribution( QPObject ):
         # 4rd column: calculate total difference in "magnitude units"
         values[:,3] = numpy.sqrt( values[:,1]**2 + values[:,2]**2 )
 
-        #print values[:,3].tolist()
-
         # select only picks with a "magnitude difference" of less than 0.1
 
         sel = ( values[:,3] < 0.1 )             # selector: TODO use magnitude threshold
@@ -521,14 +632,6 @@ class PMCProbabilityDistribution( QPObject ):
         else:
             numSample   = 0
 
-        #a = sample[:,3]
-        #a.sort(axis=0)
-        #b = a.tolist()
-        #print b
-        #print " max = ", max(b)
-        #print " min = ", min(b)
-        #print " len = ", len(b)
-
         if ( numSample < 10 ):                                                       # not enough samples available
             nonSample = values[numpy.logical_not(sel),:]                             # invert selector
 
@@ -547,15 +650,15 @@ class PMCProbabilityDistribution( QPObject ):
                 else:
                     newSample = sortedSelection[0:10,:]
                     
-                probability = newSample[:,0].sum() / 10.0  
+                probability = newSample[:,0].sum() / 10.0
                 numSample = 10
         else:
             probability = sample[:,0].sum() / numSample
-            #print sample[:,0].sum()
 
         return ( probability, numSample )
+
         
-    def fillup( self, calcMagnitudeFromDistance, override = False ):
+    def fillup( self, calcMagnitudeFromDistance, override=False ):
 
         # if self.pickInfo has only unpicked entries, set whole distro to zeros
         if self.pickInfo[:,2].sum() == 0.0:
@@ -570,16 +673,19 @@ class PMCProbabilityDistribution( QPObject ):
 
         return True
 
+
     def setSmooth( self, smooth ):
         if smooth != self.smooth:
             if self.distStyle != 0:
                 self.resetDistro()
             self.smooth = smooth
 
+
     def resetDistro( self ):
         if self.distStyle != 0:
             self.distro = numpy.ones( ( len(self.distanceGrid), len(self.magnitudeGrid) ), dtype=float ) * numpy.nan
 
+
     def restoreDistStyle( self, dist_style ):
 
         if dist_style is None:
