@@ -2,7 +2,7 @@
 # -*- coding: iso-8859-1 -*-
 #
 # quakepy/test/unitTest/QPCatalogTest.py
-# $Id: QPCatalogTest.py 157 2009-02-16 11:15:52Z fab $
+# $Id: QPCatalogTest.py 214 2009-06-25 11:45:14Z fab $
 #
 # The QuakePy package
 # http://www.quakepy.org
@@ -33,8 +33,8 @@ The QuakePy package
 http://www.quakepy.org
 """
 
-__version__  = '$Id: QPCatalogTest.py 157 2009-02-16 11:15:52Z fab $'
-__revision__ = '$Revision: 157 $'
+__version__  = '$Id: QPCatalogTest.py 214 2009-06-25 11:45:14Z fab $'
+__revision__ = '$Revision: 214 $'
 __author__   = "Fabian Euchner <fabian@fabian-euchner.de>, Danijel Schorlemmer <ds@usc.edu>"
 __license__  = "GPL"
 
@@ -601,6 +601,50 @@ class QPCatalogTest( QPTestCase ):
             # return to the original directory
             os.chdir( cwd )
 
+    def testJMADeck_JMAONLY( self ):
+        """
+        - read a catalog from JMA Deck format, import only JMA origins
+        - write catalog to QuakeML format
+        - read catalog from written QuakeML format, compare
+        """
+        print
+        print " ----- testJMADeck_JMAONLY: read JMA Deck catalogue, import only JMA origins, write as QuakeML -----"
+
+        # setup test name
+        QPTestCase.setTestName( self, "QPCatalog-JMADeck-JMAONLY" )
+
+        # cd to the test directory, remember current directory 
+        cwd = os.getcwd()
+        os.chdir( QPTestCase.TestDirPath )
+      
+        try:
+
+            infile   = 'jma.deck.test.dat'
+            outfile  = 'jma.jmaonly.deck.test.out.qml'
+            
+            # copy reference catalog file to test dir
+            shutil.copyfile( os.path.join( self.__referenceDataDir, infile ),
+                             os.path.join( QPTestCase.TestDirPath, infile ) )
+            
+            qpc = QPCatalog()
+
+            print " importing JMA Deck catalog ..."
+            qpc.importJMADeck( infile, jmaonly=True )
+            print " read uncompressed JMA Deck catalog file %s with %s events" % ( infile, qpc.size() )
+
+            print " write uncompressed QuakeML catalog file %s" % outfile
+            qpc.writeXML( outfile )
+
+            qpc2 = QPCatalog( outfile )
+            print " checking, read QuakeML catalog file %s: %s events" % ( outfile, qpc2.size() )
+
+            error = "Error: original and re-read catalog are not equal"
+            self.failIf( qpc != qpc2, error )
+
+        finally:
+            # return to the original directory
+            os.chdir( cwd )
+            
     def testGSE2_0Bulletin( self ):
         """
         - read a catalog from GSE2.0 Bulletin format
